@@ -130,7 +130,7 @@ public class Functions {
     
     public static double xMax(Geometry geometry) {
         Coordinate[] points = geometry.getCoordinates();
-        double max = Double.MIN_VALUE;
+        double max = - Double.MAX_VALUE;
         for (int i=0; i < points.length; i++) {
             max = Math.max(points[i].getX(), max);
         }
@@ -148,13 +148,35 @@ public class Functions {
     
     public static double yMax(Geometry geometry) {
         Coordinate[] points = geometry.getCoordinates();
-        double max = Double.MIN_VALUE;
+        double max = - Double.MAX_VALUE;
         for (int i=0; i < points.length; i++) {
             max = Math.max(points[i].getY(), max);
         }
         return max;
     }
 
+    public static Double zMax(Geometry geometry) {
+        Coordinate[] points = geometry.getCoordinates();
+        double max = - Double.MAX_VALUE;
+        for (int i=0; i < points.length; i++) {
+            if(java.lang.Double.isNaN(points[i].getZ()))
+                continue;
+            max = Math.max(points[i].getZ(), max);
+        }
+        return max == -Double.MAX_VALUE ? null : max;
+    }
+
+    public static Double zMin(Geometry geometry) {
+        Coordinate[] points = geometry.getCoordinates();
+        double min = Double.MAX_VALUE;
+        for(int i=0; i < points.length; i++){
+            if(java.lang.Double.isNaN(points[i].getZ()))
+                continue;
+            min = Math.min(points[i].getZ(), min);
+        }
+        return min == Double.MAX_VALUE ? null : min;
+    }
+
     public static Geometry transform(Geometry geometry, String sourceCRS, String targetCRS)
         throws FactoryException, TransformException {
         return transform(geometry, sourceCRS, targetCRS, false);
@@ -169,20 +191,20 @@ public class Functions {
         return JTS.transform(geometry, transform);
     }
 
-    private static CoordinateReferenceSystem parseCRSString(String CRSString) throws FactoryException {
+    private static CoordinateReferenceSystem parseCRSString(String CRSString)
+            throws FactoryException
+    {
         try {
-            return CRS.parseWKT(CRSString);
+            return CRS.decode(CRSString);
         }
-        catch (FactoryException ex1) {
+        catch (NoSuchAuthorityCodeException e) {
             try {
-                return CRS.decode(CRSString);
-            } catch (NoSuchAuthorityCodeException ex4) {
-                throw new NoSuchAuthorityCodeException("that Authority code cannot be found", CRSString, CRSString);
-            } catch (FactoryException ex6) {
-                throw new FactoryException("WKT format is illegal");
+                return CRS.parseWKT(CRSString);
+            }
+            catch (FactoryException ex) {
+                throw new FactoryException("First failed to read as a well-known CRS code: \n" + e.getMessage() + "\nThen failed to read as a WKT CRS string: \n" + ex.getMessage());
             }
         }
-
     }
 
     public static Geometry flipCoordinates(Geometry geometry) {
@@ -258,6 +280,24 @@ public class Functions {
         return geometry.getNumPoints();
     }
 
+    public static int nDims(Geometry geometry) {
+        int count_dimension =0;
+        Coordinate geom = geometry.getCoordinate();
+        Double x_cord = geom.getX();
+        Double y_cord = geom.getY();
+        Double z_cord = geom.getZ();
+        Double m_cord = geom.getM();
+        if(!java.lang.Double.isNaN(x_cord))
+            count_dimension++;
+        if(!java.lang.Double.isNaN(y_cord))
+            count_dimension++;
+        if(!java.lang.Double.isNaN(z_cord))
+            count_dimension++;
+        if(!java.lang.Double.isNaN(m_cord))
+            count_dimension++;
+        return count_dimension;
+    }
+
     public static int numGeometries(Geometry geometry) {
         return geometry.getNumGeometries();
     }
