@@ -26,6 +26,7 @@ import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.fs.RemoteIterator;
 import org.apache.hadoop.hdfs.HdfsConfiguration;
 import org.apache.hadoop.hdfs.MiniDFSCluster;
+import org.apache.sedona.common.utils.GeomUtils;
 import org.apache.sedona.core.TestBase;
 import org.apache.sedona.core.formatMapper.shapefileParser.ShapefileReader;
 import org.apache.sedona.core.formatMapper.shapefileParser.boundary.BoundBox;
@@ -34,7 +35,6 @@ import org.apache.sedona.core.spatialRDD.LineStringRDD;
 import org.apache.sedona.core.spatialRDD.PointRDD;
 import org.apache.sedona.core.spatialRDD.PolygonRDD;
 import org.apache.sedona.core.spatialRDD.SpatialRDD;
-import org.apache.sedona.core.utils.GeomUtils;
 import org.geotools.data.DataStore;
 import org.geotools.data.DataStoreFinder;
 import org.geotools.data.FeatureSource;
@@ -104,23 +104,14 @@ public class ShapefileReaderTest
      *
      * @throws IOException
      */
-    @Ignore
-    public void testShapefileEndWithUndefinedType()
+    @Test
+    public void testShapefileEndWithUnsupportedType()
             throws IOException
     {
-        // load shape with geotool.shapefile
-        String inputLocation = getShapeFilePath("undefined");
-        FeatureCollection<SimpleFeatureType, SimpleFeature> collection = loadFeatures(inputLocation);
-        // load shapes with our tool
+        // Read data that is in PolygonZ format
+        String inputLocation = getShapeFilePath("unsupported");
         SpatialRDD shapeRDD = ShapefileReader.readToGeometryRDD(sc, inputLocation);
-        FeatureIterator<SimpleFeature> features = collection.features();
-        int nullNum = 0;
-        while (features.hasNext()) {
-            SimpleFeature feature = features.next();
-            Geometry g = (Geometry) feature.getDefaultGeometry();
-            if (g == null) { nullNum++; }
-        }
-        assertEquals(shapeRDD.getRawSpatialRDD().count(), collection.size() - nullNum);
+        assertEquals(0, shapeRDD.getRawSpatialRDD().count());
     }
 
     /**
