@@ -58,6 +58,7 @@ object Catalog {
     function[ST_3DDistance](),
     function[ST_ConvexHull](),
     function[ST_NPoints](),
+    function[ST_NDims](),
     function[ST_Buffer](),
     function[ST_Envelope](),
     function[ST_Length](),
@@ -124,6 +125,8 @@ object Catalog {
     function[ST_PointN](),
     function[ST_AsEWKT](),
     function[ST_Force_2D](),
+    function[ST_ZMax](),
+    function[ST_ZMin](),
     function[ST_YMax](),
     function[ST_YMin](),
     function[ST_XMax](),
@@ -133,6 +136,8 @@ object Catalog {
     function[ST_CollectionExtract](),
     function[ST_Normalize](),
     function[ST_LineFromMultiPoint](),
+    function[ST_MPolyFromText](0),
+    function[ST_MLineFromText](0),
     // Expression for rasters
     function[RS_NormalizedDifference](),
     function[RS_Mean](),
