@@ -19,6 +19,9 @@
 
 package org.apache.sedona.sql
 
+import org.apache.spark.sql.catalyst.expressions.{EmptyRow, Literal}
+import org.apache.spark.sql.sedona_sql.expressions.{ST_Contains, ST_CoveredBy, ST_Covers, ST_Crosses, ST_Disjoint, ST_Equals, ST_Intersects, ST_OrderingEquals, ST_Overlaps, ST_Point, ST_Touches, ST_Within}
+
 class predicateTestScala extends TestBaseScala {
 
   describe("Sedona-SQL Predicate Test") {
@@ -50,6 +53,24 @@ class predicateTestScala extends TestBaseScala {
       var resultDf = sparkSession.sql("select * from pointdf where ST_Within(pointdf.arealandmark, ST_PolygonFromEnvelope(1.0,100.0,1000.0,1100.0))")
       assert(resultDf.count() == 999)
     }
+    it("Passed ST_Covers") {
+      var pointCsvDF = sparkSession.read.format("csv").option("delimiter", ",").option("header", "false").load(csvPointInputLocation)
+      pointCsvDF.createOrReplaceTempView("pointtable")
+      var pointDf = sparkSession.sql("select ST_Point(cast(pointtable._c0 as Decimal(24,20)), cast(pointtable._c1 as Decimal(24,20))) as arealandmark from pointtable")
+      pointDf.createOrReplaceTempView("pointdf")
+
+      var resultDf = sparkSession.sql("select * from pointdf where ST_Covers(ST_PolygonFromEnvelope(1.0,100.0,101.0,201.0), pointdf.arealandmark)")
+      assert(resultDf.count() == 100)
+    }
+    it("Passed ST_CoveredBy") {
+      var pointCsvDF = sparkSession.read.format("csv").option("delimiter", ",").option("header", "false").load(csvPointInputLocation)
+      pointCsvDF.createOrReplaceTempView("pointtable")
+      var pointDf = sparkSession.sql("select ST_Point(cast(pointtable._c0 as Decimal(24,20)), cast(pointtable._c1 as Decimal(24,20))) as arealandmark from pointtable")
+      pointDf.createOrReplaceTempView("pointdf")
+
+      var resultDf = sparkSession.sql("select * from pointdf where ST_CoveredBy(pointdf.arealandmark, ST_PolygonFromEnvelope(1.0,100.0,101.0,201.0))")
+      assert(resultDf.count() == 100)
+    }
 
     it("Passed ST_Equals for ST_Point") {
       // Select a point from the table and check if any point in the table is equal to the selected point.
@@ -209,5 +230,38 @@ class predicateTestScala extends TestBaseScala {
       assert(!notOrderEqualsDiffGeom.take(1)(0).get(0).asInstanceOf[Boolean])
       assert(!notOrderEqualsDiffOrder.take(1)(0).get(0).asInstanceOf[Boolean])
     }
+
+    it("Passed edge cases of ST_Contains and ST_Covers") {
+      val testtable = sparkSession.sql("select ST_GeomFromWKT('POLYGON((2 0, 0 2, -2 0, 2 0))') AS a, ST_GeomFromWKT('POINT(2 0)') AS b")
+      testtable.createOrReplaceTempView("testtable")
+      val contains = sparkSession.sql("select ST_Contains(a, b) from testtable").take(1)(0)
+      val covers = sparkSession.sql("select ST_Covers(a, b) from testtable").take(1)(0)
+      assert(!contains.get(0).asInstanceOf[Boolean])
+      assert(covers.get(0).asInstanceOf[Boolean])
+    }
+
+    it("Passed edge cases of ST_Within and ST_CoveredBy") {
+      val testtable = sparkSession.sql("select ST_GeomFromWKT('POLYGON((2 0, 0 2, -2 0, 2 0))') AS a, ST_GeomFromWKT('POINT(2 0)') AS b")
+      testtable.createOrReplaceTempView("testtable")
+      val within = sparkSession.sql("select ST_Within(b, a) from testtable").take(1)(0)
+      val coveredBy = sparkSession.sql("select ST_CoveredBy(b, a) from testtable").take(1)(0)
+      assert(!within.get(0).asInstanceOf[Boolean])
+      assert(coveredBy.get(0).asInstanceOf[Boolean])
+    }
+
+    Seq(
+      ST_Contains, ST_Intersects, ST_Within, ST_Covers, ST_CoveredBy, ST_Crosses,
+      ST_Overlaps, ST_Touches, ST_Equals, ST_Disjoint, ST_OrderingEquals
+    ).foreach { predicate =>
+      it(s"Passed null handling in $predicate") {
+        val point = ST_Point(Literal.create(0.0) :: Literal.create(0.0) :: Literal.create(0.0):: Nil)
+        val missing = Literal.create(null)
+
+        assert(predicate(point :: point :: Nil).eval(EmptyRow) != null)
+        assert(predicate(point :: missing :: Nil).eval(EmptyRow) == null)
+        assert(predicate(missing :: point :: Nil).eval(EmptyRow) == null)
+        assert(predicate(missing :: missing :: Nil).eval(EmptyRow) == null)
+      }
+    }
   }
 }
